#! /bin/sh
# the next line restarts using tclsh \
exec `which tclsh` $0 ${1+"$@"}

set version "v0.1"

if {0} {
WLAN-stats-generator for Kismet's .csv-files
Written 2006 by Jannis Achstetter <jannis_achstetter@web.de>

Changelog:

*v0.1:
  - Initial release

/Changelog

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
}

proc scanfiles {} {
  global verbose path
  if {[catch {set list [split [glob "$path\*.csv"]]}]} {
    puts "FATAL ERROR - I didn't find any Kismet .csv-files in \"$path\""
    exit
  } else {
    if {$verbose} {puts "OK, I found [llength $list] .csv-files in \"$path\""}
  }

  array set types {}
  array set essids {}
  array set chans {}
  array set chancounts {}
  array set encrs {}
  array set encrcounts {}
  array set rates {}
  array set ips {}
  array set ipcounts {}

  set netfcount 0
  foreach i $list {
    if {$verbose} {puts -nonewline "Reading file $i ... "}
    set fchan [open "$i" r]

    set fncount 0
    while {![eof $fchan]} {
      gets $fchan line
      set list [split $line \x3B]
      set nettype [lindex $list 1]
      set essid [lindex $list 2]
      set bssid [lindex $list 3]
      set channel [lindex $list 5]
      set encr [lindex $list 7]
      set rate [lindex $list 9]
      set firsttime [lindex $list 19]
      set lasttime [lindex $list 20]
      set ip [lindex $list 37]
      if {[string length $channel] == 1} { set channel "0$channel" }
      if {$bssid != "BSSID" && $bssid != ""} {
        array set essids [list $bssid $essid]
        array set chans [list $bssid $channel]
        array set encrs [list $bssid $encr]
        array set ips [list $bssid $ip]
        incr fncount
        #puts "BSSID: $bssid TYPE: $nettype CHAN: $channel ESSID: $essid ENCR: $encr RATE: $rate FIRSTTIME: $firsttime LASTTIME: $lasttime IP: $ip"
      }
    }
    if {$verbose} {puts "OK, found $fncount networks"}
    incr netfcount $fncount
  }

  set netcount 0
  foreach k [array names essids] {
    set channel [lindex [split [array get chans $k]] 1]
    set chancount [lindex [array get chancounts $channel] 1]
    if {$chancount == ""} { set chancount 1 } else { incr chancount }
    array set chancounts [list $channel $chancount]
    incr netcount
  }

  if {$verbose} {puts "Total $netfcount networks - $netcount unique BSSIDs"}

  if {$verbose} {puts "\nChannel-Usage-Statistics:"}
  puts "Channel\tCount\tPerc."
  foreach l [lsort [array names chancounts]] {
    set percentage [expr ([lindex [array get chancounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    puts "[lindex [array get chancounts $l] 0]\t[lindex [array get chancounts $l] 1]\t$precom\.$postcom%"
  }

  foreach k [array names encrs] {
    set encr [lindex [split [array get encrs $k]] 1]
    set encrcount [lindex [array get encrcounts $encr] 1]
    if {$encrcount == ""} { set encrcount 1 } else { incr encrcount }
    array set encrcounts [list $encr $encrcount]
  }

  if {$verbose} {puts -nonewline "\nEncryption-Statistics:"}
  puts "\nEncryption-name\t\t\tCount\tPerc."
  foreach l [lsort [array names encrcounts]] {
    set percentage [expr ([lindex [array get encrcounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    if {[string length [lindex [array get encrcounts $l] 0]] < 8} {
      puts "[lindex [array get encrcounts $l] 0]\t\t\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } elseif {[string length [lindex [array get encrcounts $l] 0]] < 16} {
      puts "[lindex [array get encrcounts $l] 0]\t\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } elseif {[string length [lindex [array get encrcounts $l] 0]] < 24} {
      puts "[lindex [array get encrcounts $l] 0]\t\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    } else {
      puts "[lindex [array get encrcounts $l] 0]\t[lindex [array get encrcounts $l] 1]\t$precom\.$postcom%"
    }
  }

  foreach k [array names ips] {
    set ip [lindex [split [array get ips $k]] 1]
    set ip "[lindex [split $ip .] 0].[lindex [split $ip .] 1].[lindex [split $ip .] 2].XXX"
    set ipcount [lindex [array get ipcounts $ip] 1]
    if {$ipcount == ""} { set ipcount 1 } else { incr ipcount }
    array set ipcounts [list $ip $ipcount]
  }

  if {$verbose} {puts -nonewline "\nIP-Usage-Statistics:"}
  puts "\nIP-Range\t\tCount\tPerc."
  foreach l [lsort [array names ipcounts]] {
    set percentage [expr ([lindex [array get ipcounts $l] 1] / ($netcount + 0.0)) * 100]
    set precom [lindex [split $percentage .] 0]
    set postcom [lindex [split $percentage .] 1]
    if {[string length $precom] == 1} {
      set precom "0$precom"
    }
    set postcom [string range $postcom 0 1]
    if {[string length $postcom] < 2} {
      append postcom 0
    }
    if {[lindex [array get ipcounts $l] 0] != "0.0.0.XXX"} {
      puts "[lindex [array get ipcounts $l] 0]\t\t[lindex [array get ipcounts $l] 1]\t$precom\.$postcom%"
    } else {
      puts "Unknown\t\t\t[lindex [array get ipcounts $l] 1]\t$precom\.$postcom%"
    }
  }
}

proc usage {} {
  global version
  puts "WLAN-stats-generator $version for Kismet's .csv-files"
  puts "Written 2006 by Jannis Achstetter <jannis_achstetter@web.de>\n"
  puts "Usage:"
  puts "  ./wlanstats \[-v\] --path=PATH"
  puts "  where -v is for verbose and"
  puts "  --path= specifies the path where the .csv-"
  puts "  files are stored, including the trailing \"\/\"\n"
  puts "Example: ./wlanstats -v --path=/tmp/"
  exit
}

if {$::argc == 0} {usage}

set verbose 0
foreach m $::argv {
  if {$m == "-v"} {set verbose 1}
  if {[lindex [split $m =] 0] == "--path"} {set path [lindex [split $m =] 1]}
}

if {[catch {string length $path}]} {
  puts "You didn't specify a path to search the .csv-files in!"
  usage
}

scanfiles